@@ -33,10 +33,16 @@ from reportlab.graphics.shapes import (
     _CLOSEPATH, Circle, Drawing, Ellipse, Group, Image, Line, Path, PolyLine,
     Polygon, Rect, SolidShape, String,
 )
-from reportlab.graphics.transform import mmult
 from reportlab.lib import colors
 from reportlab.lib.units import pica, toLength
 from reportlab.lib.utils import haveImages
+
+try:
+    from reportlab.graphics.transform import mmult
+except ImportError:
+    # Before Reportlab 3.5.61
+    from reportlab.graphics.shapes import mmult
+
 from lxml import etree
 import cssselect2
 import tinycss2
@@ -48,6 +54,7 @@ from .utils import (
 
 from .fonts import (
     get_global_font_map, DEFAULT_FONT_NAME, DEFAULT_FONT_WEIGHT, DEFAULT_FONT_STYLE,
+    DEFAULT_FONT_SIZE,
 )
 
 # To keep backward compatibility, since those functions where previously part of the svglib module
@@ -64,10 +71,10 @@ def find_font(font_name, weight='normal', style='normal'):
     return _fonts_find_font(font_name, weight, style)
 
 
-__version__ = '1.2.1'
+__version__ = '1.3.0'
 __license__ = 'LGPL 3'
 __author__ = 'Dinu Gherman'
-__date__ = '2022-01-27'
+__date__ = '2022-05-18'
 
 XML_NS = 'http://www.w3.org/XML/1998/namespace'
 
@@ -120,12 +127,12 @@ class ClippingPath(Path):
 
 
 class CSSMatcher(cssselect2.Matcher):
-    def __init__(self, style_content):
-        super().__init__()
-        self.rules = tinycss2.parse_stylesheet(
+    def add_styles(self, style_content):
+        rules = tinycss2.parse_stylesheet(
             style_content, skip_comments=True, skip_whitespace=True
         )
-        for rule in self.rules:
+
+        for rule in rules:
             if not rule.prelude:
                 continue
             selectors = cssselect2.compile_selector_list(rule.prelude)
@@ -285,7 +292,7 @@ class Svg2RlgAttributeConverter(AttributeConverter):
     def split_attr_list(attr):
         return shlex.split(attr.strip().replace(',', ' '))
 
-    def convertLength(self, svgAttr, em_base=12, attr_name=None, default=0.0):
+    def convertLength(self, svgAttr, em_base=DEFAULT_FONT_SIZE, attr_name=None, default=0.0):
         "Convert length to points."
 
         text = svgAttr.replace(',', ' ').strip()
@@ -535,6 +542,7 @@ class SvgRenderer:
         self.definitions = {}
         self.waiting_use_nodes = defaultdict(list)
         self._external_svgs = {}
+        self.attrConverter.css_rules = CSSMatcher()
 
     def render(self, svg_node):
         node = NodeTracker(svg_node)
@@ -580,7 +588,12 @@ class SvgRenderer:
             self.renderStyle(n)
         elif name == "symbol":
             item = self.renderSymbol(n)
-            parent.add(item)
+            # First time the symbol node is rendered, it should not be part of a group.
+            # It is only rendered to be part of definitions.
+            if n.attrib.get('_rendered'):
+                parent.add(item)
+            else:
+                n.set('_rendered', '1')
         elif name == "use":
             item = self.renderUse(n, clipping=clipping)
             parent.add(item)
@@ -710,7 +723,8 @@ class SvgRenderer:
             - the path to an image file for any raster image targets
             - None if any problem occurs
         """
-        xlink_href = node.attrib.get('{http://www.w3.org/1999/xlink}href')
+        # Bare 'href' was introduced in SVG 2.
+        xlink_href = node.attrib.get('{http://www.w3.org/1999/xlink}href') or node.attrib.get('href')
         if not xlink_href:
             return None
 
@@ -844,16 +858,14 @@ class SvgRenderer:
 
         return group
 
-    def renderG(self, node, clipping=None, display=1):
+    def renderG(self, node, clipping=None):
         getAttr = node.getAttribute
         id, transform = map(getAttr, ("id", "transform"))
         gr = Group()
         if clipping:
             gr.add(clipping)
         for child in node.getchildren():
-            item = self.renderNode(child, parent=gr)
-            if item and display:
-                gr.add(item)
+            self.renderNode(child, parent=gr)
 
         if transform:
             self.shape_converter.applyTransformOnGroup(transform, gr)
@@ -861,10 +873,10 @@ class SvgRenderer:
         return gr
 
     def renderStyle(self, node):
-        self.attrConverter.css_rules = CSSMatcher(node.text)
+        self.attrConverter.css_rules.add_styles(node.text or "")
 
     def renderSymbol(self, node):
-        return self.renderG(node, display=0)
+        return self.renderG(node)
 
     def renderA(self, node):
         # currently nothing but a group...
@@ -946,7 +958,7 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
             group.add(shape)
             return group
 
-    def convert_length_attrs(self, node, *attrs, em_base=None, **kwargs):
+    def convert_length_attrs(self, node, *attrs, em_base=DEFAULT_FONT_SIZE, **kwargs):
         # Support node both as NodeTracker or lxml node
         getAttr = (
             node.getAttribute if hasattr(node, 'getAttribute')
@@ -960,8 +972,9 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
         ]
 
     def convertLine(self, node):
-        x1, y1, x2, y2 = self.convert_length_attrs(node, 'x1', 'y1', 'x2', 'y2')
-        return Line(x1, y1, x2, y2)
+        points = self.convert_length_attrs(node, 'x1', 'y1', 'x2', 'y2')
+        nudge_points(points)
+        return Line(*points)
 
     def convertRect(self, node):
         x, y, width, height, rx, ry = self.convert_length_attrs(
@@ -971,6 +984,10 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
             rx = width / 2
         if ry > (height / 2):
             ry = height / 2
+        if rx and not ry:
+            ry = rx
+        elif ry and not rx:
+            rx = ry
         return Rect(x, y, width, height, rx=rx, ry=ry)
 
     def convertCircle(self, node):
@@ -992,6 +1009,7 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
             # Odd number of coordinates or no coordinates, invalid polyline
             return None
 
+        nudge_points(points)
         polyline = PolyLine(points)
         self.applyStyleOnShape(polyline, node)
         has_fill = self.attrConverter.findAttr(node, 'fill') not in ('', 'none')
@@ -1017,6 +1035,7 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
         if len(points) % 2 != 0 or len(points) == 0:
             # Odd number of coordinates or no coordinates, invalid polygon
             return None
+        nudge_points(points)
         shape = Polygon(points)
 
         return shape
@@ -1053,7 +1072,7 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
         fw = attrConv.findAttr(node, "font-weight") or DEFAULT_FONT_WEIGHT
         fstyle = attrConv.findAttr(node, "font-style") or DEFAULT_FONT_STYLE
         ff = attrConv.convertFontFamily(ff, fw, fstyle)
-        fs = attrConv.findAttr(node, "font-size") or "12"
+        fs = attrConv.findAttr(node, "font-size") or str(DEFAULT_FONT_SIZE)
         fs = attrConv.convertLength(fs)
         x, y = self.convert_length_attrs(node, 'x', 'y', em_base=fs)
         for c in itertools.chain([node], node.getchildren()):
@@ -1366,7 +1385,7 @@ class Svg2RlgShapeConverter(SvgShapeConverter):
                 "fontName", "convertFontFamily",
                 [DEFAULT_FONT_NAME, DEFAULT_FONT_WEIGHT, DEFAULT_FONT_STYLE]
             ),
-            (["font-size"], "fontSize", "convertLength", ["12"]),
+            (["font-size"], "fontSize", "convertLength", [str(DEFAULT_FONT_SIZE)]),
             (["text-anchor"], "textAnchor", "id", ["start"]),
         )
 
@@ -1455,6 +1474,26 @@ def svg2rlg(path, resolve_entities=False, **kwargs):
     return drawing
 
 
+def nudge_points(points):
+    """ Nudge first coordinate if all coordinate pairs are identical.
+
+    This works around reportlab's decision to hide shapes of size zero, even
+    when the stroke should be visible.
+    """
+    if not points:
+        return
+    if len(points) < 4:
+        return
+    x = points[0]
+    y = points[1]
+    for i in range(2, len(points)-1, 2):
+        if x != points[i] or y != points[i+1]:
+            break
+    else:
+        # All points were identical, so we nudge.
+        points[0] *= 1.0000001
+
+
 def load_svg_file(path, resolve_entities=False):
     parser = etree.XMLParser(
         remove_comments=True, recover=True, resolve_entities=resolve_entities
